/*
 * Copyright 2008 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _JOBOBJECT_H
#define _JOBOBJECT_H

#include <string>

#include <qpid/management/Manageable.h>
#include <qpid/management/ManagementObject.h>
#include <qpid/agent/ManagementAgent.h>

#include <qpid/framing/FieldTable.h>

#include "Job.h"

#include "ArgsJobGetAd.h"
#include "ArgsJobSetAttribute.h"
#include "ArgsJobHold.h"
#include "ArgsJobRelease.h"
#include "ArgsJobRemove.h"
#include "ArgsJobFetch.h"

#include "SchedulerObject.h"
#include "SubmitterObject.h"

#include "condor_classad.h"


namespace mrg {
namespace grid {

using namespace qpid::management;

class JobObject : public Manageable
{

public:

	JobObject(ManagementAgent *agent,
			  SchedulerObject *_scheduler,
			  uint32_t _clusterId,
			  uint32_t _procId,
			  const SubmitterObject *submitter);
	~JobObject();

	void update(const ClassAd &ad);

	ManagementObject *GetManagementObject(void) const;

	status_t ManagementMethod(uint32_t methodId, Args &args, std::string &text);


private:

	qmf::mrg::grid::Job *mgmtObject;
	uint32_t clusterId;
	uint32_t procId;

	status_t GetAd(qpid::framing::FieldTable &fieldTable, std::string &/*text*/);

	status_t SetAttribute(std::string name,
						  std::string value,
						  std::string &text);

	status_t Hold(std::string &reason, std::string &text);

	status_t Release(std::string &reason, std::string &text);

	status_t Remove(std::string &reason, std::string &text);

	status_t Fetch(std::string &file,
				   int32_t start,
				   int32_t end,
				   std::string &data,
				   std::string &text);
};

}} /* mrg::grid */

#endif /* _JOBOBJECT_H */
