/*
 * Copyright 2008 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _MGMTSCHEDDPLUGIN_H
#define _MGMTSCHEDDPLUGIN_H

#include "JobObject.h"
#include "SchedulerObject.h"
#include "SubmitterObject.h"

#include "SubmitterUtils.h"

#include "condor_qmgr.h"

#include "../condor_schedd.V6/ScheddPlugin.h"
#include "ClassAdLogPlugin.h"

#include "../condor_daemon_core.V6/condor_daemon_core.h"

#include <set>
#include <string>

#include "PROC_ID_comparator.h"

using namespace qpid::management;
using namespace qpid::framing;
using namespace qmf::mrg;
using namespace qmf::mrg::grid;
using namespace mrg::grid;

namespace mrg {
namespace grid {

// BIG NOTE: If Service is not first in the parent list the
//           processDirtyJobs handler will segfault when using the
//           dirtyJobs list
struct MgmtScheddPlugin : public Service, ClassAdLogPlugin, ScheddPlugin
{

public:

	typedef HashTable<PROC_ID, JobObject *> JobTableType;

	void earlyInitialize();

	void initialize();

	void shutdown();

	void update(int cmd, const ClassAd *ad);

	void archive(const ClassAd *ad);

	void newClassAd(const char */*key*/);

	void setAttribute(const char *key,
					  const char *name,
					  const char *value);

	void destroyClassAd(const char *key);

	void deleteAttribute(const char *key,
						 const char *name);

	JobTableType *getJobTable();


private:

		// ManagementAgent::Singleton cleans up the ManagementAgent
		// instance if there are no ManagementAgent::Singleton's in
		// scope!
	ManagementAgent::Singleton *singleton;

	JobTableType *jobTable;
	typedef std::set<std::string, PROC_ID_comparator> DirtyJobsType;
	DirtyJobsType *dirtyJobs;
	SchedulerObject *scheduler;
	typedef HashTable<MyString, SubmitterObject *> SubmitterHashTable;
	SubmitterHashTable *submitterAds;

	bool isHandlerRegistered;


	int HandleMgmtSocket(Service *, Stream *);

	void processDirtyJobs();

	void markDirty(const char *key);

	bool GetSubmitter(MyString &name, SubmitterObject *&submitter);

};

}} /* mrg::grid */

#endif /* _MGMTSCHEDDPLUGIN_H */
