/*
 * Copyright 2008 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _CONDOR_ALLOW_OPEN
#include "condor_common.h"

#include "../condor_startd.V6/StartdPlugin.h"

#include "../condor_collector.V6/hashkey.h"

#include "../condor_daemon_core.V6/condor_daemon_core.h"

#include "condor_config.h"

#include "SlotObject.h"

using namespace std;
using namespace mrg::grid;


struct MgmtStartdPlugin : public Service, StartdPlugin
{
		// ManagementAgent::Singleton cleans up the ManagementAgent
		// instance if there are no ManagementAgent::Singleton's in
		// scope!
	ManagementAgent::Singleton *singleton;

	typedef HashTable<AdNameHashKey, SlotObject *> SlotHashTable;

	SlotHashTable *startdAds;

	void
	initialize()
	{
		char *host;
		int port;
		char *tmp;
		string storefile;

		dprintf(D_FULLDEBUG, "MgmtStartdPlugin: Initializing...\n");

		singleton = new ManagementAgent::Singleton();

		startdAds = new SlotHashTable(4096, &adNameHashFunction);

		ManagementAgent *agent = singleton->getInstance();

		Slot::registerSelf(agent);

		port = param_integer("QMF_BROKER_PORT", 5672);
		if (NULL == (host = param("QMF_BROKER_HOST"))) {
			host = strdup("localhost");
		}

		tmp = param("QMF_STOREFILE");
		if (NULL == tmp) {
			storefile = ".startd_storefile";
		} else {
			storefile = tmp;
			free(tmp); tmp = NULL;
		}
		
		agent->init(string(host), port,
					param_integer("QMF_UPDATE_INTERVAL", 10),
					true,
					storefile);

		free(host);
	}

	void
	shutdown()
	{
		if (!param_boolean("QMF_DELETE_ON_SHUTDOWN", false)) {
			return;
		}

		dprintf(D_FULLDEBUG, "MgmtStartdPlugin: shutting down...\n");
		if (singleton) {
			delete singleton;
			singleton = NULL;
		}
	}

	void
	update(const ClassAd *ad, const ClassAd *)
	{
		AdNameHashKey hashKey;
		SlotObject *slotObject;

		if (!makeStartdAdHashKey(hashKey, ((ClassAd *) ad), NULL)) {
			dprintf(D_FULLDEBUG, "Could not make hashkey -- ignoring ad\n");
		}

		if (startdAds->lookup(hashKey, slotObject)) {
				// Key doesn't exist
			slotObject = new SlotObject(singleton->getInstance());

				// Ignore old value, if it existed (returned)
			startdAds->insert(hashKey, slotObject);
		}

		slotObject->update(*ad);
	}

	void
	invalidate(const ClassAd */*ad*/)
	{
			//printf("Invalidate\n");
	}
};

static MgmtStartdPlugin instance;
