/*
 * Copyright 2008 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _CONDOR_ALLOW_OPEN
#include "condor_common.h"

#include "Utils.h"

#include "condor_classad.h"

#include "condor_debug.h"


enum AttributeType {
	EXPR_TYPE = 0,
	INTEGER_TYPE = 1,
	FLOAT_TYPE = 2,
	STRING_TYPE = 3
};


static const char *TYPE_ATTR = "TYPE";
static const char *VALUE_ATTR = "VALUE";

using namespace qpid::framing;


bool
PopulateFieldTableFromAd(ClassAd &ad,
						 qpid::framing::FieldTable &fieldTable)
{
	ExprTree *expr;
	char *name;
	char *value;
	qpid::framing::FieldTable attr;

	ad.ResetExpr();
	while (NULL != (expr = ad.NextExpr())) {
		attr = qpid::framing::FieldTable();
		name = ((Variable *) expr->LArg())->Name();

			// All these extra lookups are horrible. They have to
			// be there because the ClassAd may have multiple
			// copies of the same attribute name! This means that
			// the last attribute with a given name will set the
			// value, but the last attribute tends to be the
			// attribute with the oldest (wrong) value. How
			// annoying is that!
		if (!(expr = ad.Lookup(name))) {
			dprintf(D_ALWAYS, "Failed to lookup %s\n", name);

			return false;
		}

		switch (expr->RArg()->MyType()) {
		case LX_INTEGER:
			attr.setInt(TYPE_ATTR, INTEGER_TYPE);
			break;
		case LX_FLOAT:
			attr.setInt(TYPE_ATTR, FLOAT_TYPE);
			break;
		case LX_STRING:
			attr.setInt(TYPE_ATTR, STRING_TYPE);
			break;
		default:
			attr.setInt(TYPE_ATTR, EXPR_TYPE);
		}

		expr->RArg()->PrintToNewStr(&value);
		attr.setString(VALUE_ATTR, value);
		free(value);

		fieldTable.setTable(name, attr);
	}

	return true;
}


bool
PopulateAdFromFieldTable(ClassAd &ad,
						 FieldTable &fieldTable)
{
	for (FieldTable::ValueMap::const_iterator entry = fieldTable.begin();
		 fieldTable.end() != entry;
		 entry++) {
		const char *name = entry->first.c_str();
		const char *value = NULL;
		FieldTable attr;

		if (!fieldTable.getTable(name, attr)) {
			return false;
		}

		value = attr.getAsString(VALUE_ATTR).c_str();

		switch (attr.getAsInt(TYPE_ATTR)) {
		case INTEGER_TYPE:
			ad.Assign(name, strtol(value, NULL, 10));
			break;
		case FLOAT_TYPE:
			ad.Assign(name, (float) strtod(value, NULL));
			break;
// TODO: Fix potential memory leaks below			
		case STRING_TYPE:
			ad.Assign(name, strdup(value));
			break;
		case EXPR_TYPE:
			ad.AssignExpr(name, strdup(value));
			break;
		}
	}

	return true;
}
